Struct OverlappingState

struct OverlappingState { ... }

Represents the current state of an overlapping search.

This is used for overlapping searches since they need to know something about the previous search. For example, when multiple patterns match at the same position, this state tracks the last reported pattern so that the next search knows whether to report another matching pattern or continue with the search at the next position. Additionally, it also tracks which state the last search call terminated in.

This type provides little introspection capabilities. The only thing a caller can do is construct it and pass it around to permit search routines to use it to track state, and also ask whether a match has been found.

Callers should always provide a fresh state constructed via OverlappingState::start when starting a new search. Reusing state from a previous search may result in incorrect results.

Implementations

impl OverlappingState

fn start() -> OverlappingState

Create a new overlapping state that begins at the start state of any automaton.

fn get_match(self: &Self) -> Option<HalfMatch>

Return the match result of the most recent search to execute with this state.

A searches will clear this result automatically, such that if no match is found, this will correctly report None.

impl Clone for OverlappingState

fn clone(self: &Self) -> OverlappingState

impl Debug for OverlappingState

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Eq for OverlappingState

impl Freeze for OverlappingState

impl PartialEq for OverlappingState

fn eq(self: &Self, other: &OverlappingState) -> bool

impl RefUnwindSafe for OverlappingState

impl Send for OverlappingState

impl StructuralPartialEq for OverlappingState

impl Sync for OverlappingState

impl Unpin for OverlappingState

impl UnsafeUnpin for OverlappingState

impl UnwindSafe for OverlappingState

impl<T> Any for OverlappingState

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for OverlappingState

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for OverlappingState

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for OverlappingState

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for OverlappingState

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for OverlappingState

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for OverlappingState

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for OverlappingState

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for OverlappingState

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>