Enum StartError

enum StartError

An error that can occur when computing the start state for a search.

Computing a start state can fail for a few reasons, either based on incorrect configuration or even based on whether the look-behind byte triggers a quit state. Typically one does not need to handle this error if you're using DFA::start_state_forward (or its reverse counterpart), as that routine automatically converts StartError to a MatchError for you.

This error may be returned by the DFA::start_state routine.

This error implements the std::error::Error trait when the std feature is enabled.

This error is marked as non-exhaustive. New variants may be added in a semver compatible release.

Variants

Cache { err: CacheError }

An error that occurs when cache inefficiency has dropped below the configured heuristic thresholds.

Quit { byte: u8 }

An error that occurs when a starting configuration's look-behind byte is in this DFA's quit set.

UnsupportedAnchored { mode: crate::util::search::Anchored }

An error that occurs when the caller requests an anchored mode that isn't supported by the DFA.

Implementations

impl Clone for StartError

fn clone(self: &Self) -> StartError

impl Debug for StartError

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Display for StartError

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Error for StartError

fn source(self: &Self) -> Option<&dyn Error + 'static>

impl Freeze for StartError

impl RefUnwindSafe for StartError

impl Send for StartError

impl Sync for StartError

impl Unpin for StartError

impl UnsafeUnpin for StartError

impl UnwindSafe for StartError

impl<T> Any for StartError

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for StartError

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for StartError

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for StartError

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for StartError

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for StartError

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T> ToString for StartError

fn to_string(self: &Self) -> String

impl<T, U> Into for StartError

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for StartError

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for StartError

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>