Struct Split

struct Split<'r, 'h> { ... }

Yields all substrings delimited by a regular expression match.

The spans correspond to the offsets between matches.

The lifetime parameters are as follows:

This iterator can be created with the Regex::split method.

Implementations

impl<'r, 'h> Split<'r, 'h>

fn input<'s>(self: &'s Self) -> &'s Input<'h>

Returns the current Input associated with this iterator.

The start position on the given Input may change during iteration, but all other values are guaranteed to remain invariant.

impl<'r, 'h> Debug for Split<'r, 'h>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'r, 'h> Freeze for Split<'r, 'h>

impl<'r, 'h> FusedIterator for Split<'r, 'h>

impl<'r, 'h> Iterator for Split<'r, 'h>

fn next(self: &mut Self) -> Option<Span>

impl<'r, 'h> RefUnwindSafe for Split<'r, 'h>

impl<'r, 'h> Send for Split<'r, 'h>

impl<'r, 'h> Sync for Split<'r, 'h>

impl<'r, 'h> Unpin for Split<'r, 'h>

impl<'r, 'h> UnsafeUnpin for Split<'r, 'h>

impl<'r, 'h> UnwindSafe for Split<'r, 'h>

impl<I> IntoIterator for Split<'r, 'h>

fn into_iter(self: Self) -> I

impl<T> Any for Split<'r, 'h>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for Split<'r, 'h>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for Split<'r, 'h>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for Split<'r, 'h>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for Split<'r, 'h>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for Split<'r, 'h>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for Split<'r, 'h>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>