Struct TryCapturesMatches
struct TryCapturesMatches<'r, 'c, 'h> { ... }
An iterator over all non-overlapping leftmost matches, with their capturing groups, for a fallible search.
The iterator yields a Result<Captures, MatchError> value until no more
matches could be found.
The lifetime parameters are as follows:
'rrepresents the lifetime of the BoundedBacktracker.'crepresents the lifetime of the BoundedBacktracker's cache.'hrepresents the lifetime of the haystack being searched.
This iterator can be created with the
BoundedBacktracker::try_captures_iter method.
Implementations
impl<'r, 'c, 'h> Debug for TryCapturesMatches<'r, 'c, 'h>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'r, 'c, 'h> Freeze for TryCapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Iterator for TryCapturesMatches<'r, 'c, 'h>
fn next(self: &mut Self) -> Option<Result<Captures, MatchError>>
impl<'r, 'c, 'h> RefUnwindSafe for TryCapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Send for TryCapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Sync for TryCapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Unpin for TryCapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> UnsafeUnpin for TryCapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> UnwindSafe for TryCapturesMatches<'r, 'c, 'h>
impl<I> IntoIterator for TryCapturesMatches<'r, 'c, 'h>
fn into_iter(self: Self) -> I
impl<T> Any for TryCapturesMatches<'r, 'c, 'h>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for TryCapturesMatches<'r, 'c, 'h>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for TryCapturesMatches<'r, 'c, 'h>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for TryCapturesMatches<'r, 'c, 'h>
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for TryCapturesMatches<'r, 'c, 'h>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for TryCapturesMatches<'r, 'c, 'h>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for TryCapturesMatches<'r, 'c, 'h>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>