Struct TryFindMatches

struct TryFindMatches<'r, 'c, 'h> { ... }

An iterator over all non-overlapping matches for a fallible search.

The iterator yields a Result<Match, MatchError value until no more matches could be found.

The lifetime parameters are as follows:

This iterator can be created with the BoundedBacktracker::try_find_iter method.

Implementations

impl<'r, 'c, 'h> Debug for TryFindMatches<'r, 'c, 'h>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'r, 'c, 'h> Freeze for TryFindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Iterator for TryFindMatches<'r, 'c, 'h>

fn next(self: &mut Self) -> Option<Result<Match, MatchError>>

impl<'r, 'c, 'h> RefUnwindSafe for TryFindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Send for TryFindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Sync for TryFindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Unpin for TryFindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> UnsafeUnpin for TryFindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> UnwindSafe for TryFindMatches<'r, 'c, 'h>

impl<I> IntoIterator for TryFindMatches<'r, 'c, 'h>

fn into_iter(self: Self) -> I

impl<T> Any for TryFindMatches<'r, 'c, 'h>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for TryFindMatches<'r, 'c, 'h>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for TryFindMatches<'r, 'c, 'h>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for TryFindMatches<'r, 'c, 'h>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for TryFindMatches<'r, 'c, 'h>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for TryFindMatches<'r, 'c, 'h>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for TryFindMatches<'r, 'c, 'h>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>