Struct CapturesMatches
struct CapturesMatches<'r, 'c, 'h> { ... }
An iterator over all non-overlapping leftmost matches, with their capturing groups, for a particular search.
The iterator yields a Captures value until no more matches could be
found.
The lifetime parameters are as follows:
'rrepresents the lifetime of the PikeVM.'crepresents the lifetime of the PikeVM's cache.'hrepresents the lifetime of the haystack being searched.
This iterator can be created with the PikeVM::captures_iter method.
Implementations
impl<'r, 'c, 'h> Debug for CapturesMatches<'r, 'c, 'h>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'r, 'c, 'h> Freeze for CapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Iterator for CapturesMatches<'r, 'c, 'h>
fn next(self: &mut Self) -> Option<Captures>
impl<'r, 'c, 'h> RefUnwindSafe for CapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Send for CapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Sync for CapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> Unpin for CapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> UnsafeUnpin for CapturesMatches<'r, 'c, 'h>
impl<'r, 'c, 'h> UnwindSafe for CapturesMatches<'r, 'c, 'h>
impl<I> IntoIterator for CapturesMatches<'r, 'c, 'h>
fn into_iter(self: Self) -> I
impl<T> Any for CapturesMatches<'r, 'c, 'h>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for CapturesMatches<'r, 'c, 'h>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for CapturesMatches<'r, 'c, 'h>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for CapturesMatches<'r, 'c, 'h>
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for CapturesMatches<'r, 'c, 'h>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for CapturesMatches<'r, 'c, 'h>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for CapturesMatches<'r, 'c, 'h>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>