Struct FindMatches

struct FindMatches<'r, 'c, 'h> { ... }

An iterator over all non-overlapping matches for a particular search.

The iterator yields a Match value until no more matches could be found.

The lifetime parameters are as follows:

This iterator can be created with the PikeVM::find_iter method.

Implementations

impl<'r, 'c, 'h> Debug for FindMatches<'r, 'c, 'h>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'r, 'c, 'h> Freeze for FindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Iterator for FindMatches<'r, 'c, 'h>

fn next(self: &mut Self) -> Option<Match>

impl<'r, 'c, 'h> RefUnwindSafe for FindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Send for FindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Sync for FindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> Unpin for FindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> UnsafeUnpin for FindMatches<'r, 'c, 'h>

impl<'r, 'c, 'h> UnwindSafe for FindMatches<'r, 'c, 'h>

impl<I> IntoIterator for FindMatches<'r, 'c, 'h>

fn into_iter(self: Self) -> I

impl<T> Any for FindMatches<'r, 'c, 'h>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for FindMatches<'r, 'c, 'h>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for FindMatches<'r, 'c, 'h>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for FindMatches<'r, 'c, 'h>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for FindMatches<'r, 'c, 'h>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for FindMatches<'r, 'c, 'h>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for FindMatches<'r, 'c, 'h>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>