Struct BuildError

struct BuildError { ... }

An error that can occurred during the construction of a thompson NFA.

This error does not provide many introspection capabilities. There are generally only two things you can do with it:

Otherwise, errors typically occur when a limit has been breeched. For example, if the total heap usage of the compiled NFA exceeds the limit set by Config::nfa_size_limit, then building the NFA will fail.

Implementations

impl BuildError

fn size_limit(self: &Self) -> Option<usize>

If this error occurred because the NFA exceeded the configured size limit before being built, then this returns the configured size limit.

The limit returned is what was configured, and corresponds to the maximum amount of heap usage in bytes.

impl Clone for BuildError

fn clone(self: &Self) -> BuildError

impl Debug for BuildError

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Display for BuildError

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Error for BuildError

fn source(self: &Self) -> Option<&dyn Error + 'static>

impl Freeze for BuildError

impl RefUnwindSafe for BuildError

impl Send for BuildError

impl Sync for BuildError

impl Unpin for BuildError

impl UnsafeUnpin for BuildError

impl UnwindSafe for BuildError

impl<T> Any for BuildError

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for BuildError

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for BuildError

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for BuildError

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for BuildError

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> ToOwned for BuildError

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T> ToString for BuildError

fn to_string(self: &Self) -> String

impl<T, U> Into for BuildError

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for BuildError

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for BuildError

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>