Struct ByteClassElements
struct ByteClassElements<'a> { ... }
An iterator over all elements in an equivalence class.
This is created by the ByteClasses::elements method.
The lifetime 'a refers to the lifetime of the byte classes that this
iterator was created from.
Implementations
impl<'a> Debug for ByteClassElements<'a>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'a> Freeze for ByteClassElements<'a>
impl<'a> Iterator for ByteClassElements<'a>
fn next(self: &mut Self) -> Option<Unit>
impl<'a> RefUnwindSafe for ByteClassElements<'a>
impl<'a> Send for ByteClassElements<'a>
impl<'a> Sync for ByteClassElements<'a>
impl<'a> Unpin for ByteClassElements<'a>
impl<'a> UnsafeUnpin for ByteClassElements<'a>
impl<'a> UnwindSafe for ByteClassElements<'a>
impl<I> IntoIterator for ByteClassElements<'a>
fn into_iter(self: Self) -> I
impl<T> Any for ByteClassElements<'a>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for ByteClassElements<'a>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for ByteClassElements<'a>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for ByteClassElements<'a>
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for ByteClassElements<'a>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for ByteClassElements<'a>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for ByteClassElements<'a>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>