Struct ByteClassIter

struct ByteClassIter<'a> { ... }

An iterator over each equivalence class.

The last element in this iterator always corresponds to Unit::eoi.

This is created by the ByteClasses::iter method.

The lifetime 'a refers to the lifetime of the byte classes that this iterator was created from.

Implementations

impl<'a> Debug for ByteClassIter<'a>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'a> Freeze for ByteClassIter<'a>

impl<'a> Iterator for ByteClassIter<'a>

fn next(self: &mut Self) -> Option<Unit>

impl<'a> RefUnwindSafe for ByteClassIter<'a>

impl<'a> Send for ByteClassIter<'a>

impl<'a> Sync for ByteClassIter<'a>

impl<'a> Unpin for ByteClassIter<'a>

impl<'a> UnsafeUnpin for ByteClassIter<'a>

impl<'a> UnwindSafe for ByteClassIter<'a>

impl<I> IntoIterator for ByteClassIter<'a>

fn into_iter(self: Self) -> I

impl<T> Any for ByteClassIter<'a>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for ByteClassIter<'a>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for ByteClassIter<'a>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for ByteClassIter<'a>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for ByteClassIter<'a>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for ByteClassIter<'a>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for ByteClassIter<'a>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>