Struct GroupInfoAllNames

struct GroupInfoAllNames<'a> { ... }

An iterator over capturing groups and their names for a GroupInfo.

This iterator is created by GroupInfo::all_names.

The lifetime parameter 'a refers to the lifetime of the GroupInfo from which this iterator was created.

Implementations

impl<'a> Debug for GroupInfoAllNames<'a>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'a> Freeze for GroupInfoAllNames<'a>

impl<'a> Iterator for GroupInfoAllNames<'a>

fn next(self: &mut Self) -> Option<(PatternID, usize, Option<&'a str>)>

impl<'a> RefUnwindSafe for GroupInfoAllNames<'a>

impl<'a> Send for GroupInfoAllNames<'a>

impl<'a> Sync for GroupInfoAllNames<'a>

impl<'a> Unpin for GroupInfoAllNames<'a>

impl<'a> UnsafeUnpin for GroupInfoAllNames<'a>

impl<'a> UnwindSafe for GroupInfoAllNames<'a>

impl<I> IntoIterator for GroupInfoAllNames<'a>

fn into_iter(self: Self) -> I

impl<T> Any for GroupInfoAllNames<'a>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for GroupInfoAllNames<'a>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for GroupInfoAllNames<'a>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for GroupInfoAllNames<'a>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for GroupInfoAllNames<'a>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for GroupInfoAllNames<'a>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for GroupInfoAllNames<'a>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>