Struct DebugByte
struct DebugByte(2286)
Provides a convenient Debug implementation for a u8.
The Debug impl treats the byte as an ASCII, and emits a human readable
representation of it. If the byte isn't ASCII, then it's emitted as a hex
escape sequence.
Implementations
impl Clone for DebugByte
fn clone(self: &Self) -> DebugByte
impl Copy for DebugByte
impl Debug for DebugByte
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Freeze for DebugByte
impl RefUnwindSafe for DebugByte
impl Send for DebugByte
impl Sync for DebugByte
impl Unpin for DebugByte
impl UnsafeUnpin for DebugByte
impl UnwindSafe for DebugByte
impl<T> Any for DebugByte
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for DebugByte
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for DebugByte
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for DebugByte
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for DebugByte
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for DebugByte
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for DebugByte
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for DebugByte
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for DebugByte
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>