Struct DebugHaystack
struct DebugHaystack<'a>(2309)
Provides a convenient Debug implementation for &[u8].
This generally works best when the bytes are presumed to be mostly UTF-8, but will work for anything. For any bytes that aren't UTF-8, they are emitted as hex escape sequences.
Implementations
impl<'a> Debug for DebugHaystack<'a>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'a> Freeze for DebugHaystack<'a>
impl<'a> RefUnwindSafe for DebugHaystack<'a>
impl<'a> Send for DebugHaystack<'a>
impl<'a> Sync for DebugHaystack<'a>
impl<'a> Unpin for DebugHaystack<'a>
impl<'a> UnsafeUnpin for DebugHaystack<'a>
impl<'a> UnwindSafe for DebugHaystack<'a>
impl<T> Any for DebugHaystack<'a>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for DebugHaystack<'a>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for DebugHaystack<'a>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for DebugHaystack<'a>
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for DebugHaystack<'a>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for DebugHaystack<'a>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for DebugHaystack<'a>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>