Struct CapturesIter
struct CapturesIter<'h, F>(_)
An iterator over all non-overlapping captures for an infallible search.
The iterator yields a Captures value until no more matches could be
found.
The type parameters are as follows:
Frepresents the type of a closure that executes the search.
The lifetime parameters come from the Input type:
'his the lifetime of the underlying haystack.
When possible, prefer the iterators defined on the regex engine you're using. This tries to abstract over the regex engine and is thus a bit more unwieldy to use.
This iterator is created by Searcher::into_captures_iter and then
calling TryCapturesIter::infallible.
Implementations
impl<'h, F> Freeze for CapturesIter<'h, F>
impl<'h, F> Iterator for CapturesIter<'h, F>
fn next(self: &mut Self) -> Option<Captures>
impl<'h, F> RefUnwindSafe for CapturesIter<'h, F>
impl<'h, F> Send for CapturesIter<'h, F>
impl<'h, F> Sync for CapturesIter<'h, F>
impl<'h, F> Unpin for CapturesIter<'h, F>
impl<'h, F> UnsafeUnpin for CapturesIter<'h, F>
impl<'h, F> UnwindSafe for CapturesIter<'h, F>
impl<'h, F: $crate::fmt::Debug> Debug for CapturesIter<'h, F>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<I> IntoIterator for CapturesIter<'h, F>
fn into_iter(self: Self) -> I
impl<T> Any for CapturesIter<'h, F>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for CapturesIter<'h, F>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for CapturesIter<'h, F>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for CapturesIter<'h, F>
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for CapturesIter<'h, F>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for CapturesIter<'h, F>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for CapturesIter<'h, F>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>