Struct MatchesIter

struct MatchesIter<'h, F>(_)

An iterator over all non-overlapping matches for an infallible search.

The iterator yields a Match value until no more matches could be found.

The type parameters are as follows:

The lifetime parameters come from the Input type:

When possible, prefer the iterators defined on the regex engine you're using. This tries to abstract over the regex engine and is thus a bit more unwieldy to use.

This iterator is created by Searcher::into_matches_iter and then calling TryMatchesIter::infallible.

Implementations

impl<'h, F> MatchesIter<'h, F>

fn input<'i>(self: &'i Self) -> &'i Input<'h>

Returns the current Input used by this iterator.

The Input returned is generally equivalent to the one used to construct this iterator, but its start position may be different to reflect the start of the next search to be executed.

impl<'h, F> Freeze for MatchesIter<'h, F>

impl<'h, F> Iterator for MatchesIter<'h, F>

fn next(self: &mut Self) -> Option<Match>

impl<'h, F> RefUnwindSafe for MatchesIter<'h, F>

impl<'h, F> Send for MatchesIter<'h, F>

impl<'h, F> Sync for MatchesIter<'h, F>

impl<'h, F> Unpin for MatchesIter<'h, F>

impl<'h, F> UnsafeUnpin for MatchesIter<'h, F>

impl<'h, F> UnwindSafe for MatchesIter<'h, F>

impl<'h, F: $crate::fmt::Debug> Debug for MatchesIter<'h, F>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<I> IntoIterator for MatchesIter<'h, F>

fn into_iter(self: Self) -> I

impl<T> Any for MatchesIter<'h, F>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for MatchesIter<'h, F>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for MatchesIter<'h, F>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for MatchesIter<'h, F>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for MatchesIter<'h, F>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for MatchesIter<'h, F>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for MatchesIter<'h, F>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>