Struct TryCapturesIter
struct TryCapturesIter<'h, F> { ... }
An iterator over all non-overlapping captures for a fallible search.
The iterator yields a Result<Captures, MatchError> value until no more
matches could be found.
The type parameters are as follows:
Frepresents the type of a closure that executes the search.
The lifetime parameters come from the Input type:
'his the lifetime of the underlying haystack.
When possible, prefer the iterators defined on the regex engine you're using. This tries to abstract over the regex engine and is thus a bit more unwieldy to use.
This iterator is created by Searcher::into_captures_iter.
Implementations
impl<'h, F> TryCapturesIter<'h, F>
fn infallible(self: Self) -> CapturesIter<'h, F>Return an infallible version of this iterator.
Any item yielded that corresponds to an error results in a panic. This is useful if your underlying regex engine is configured in a way that it is guaranteed to never return an error.
impl<'h, F> Debug for TryCapturesIter<'h, F>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<'h, F> Freeze for TryCapturesIter<'h, F>
impl<'h, F> Iterator for TryCapturesIter<'h, F>
fn next(self: &mut Self) -> Option<Result<Captures, MatchError>>
impl<'h, F> RefUnwindSafe for TryCapturesIter<'h, F>
impl<'h, F> Send for TryCapturesIter<'h, F>
impl<'h, F> Sync for TryCapturesIter<'h, F>
impl<'h, F> Unpin for TryCapturesIter<'h, F>
impl<'h, F> UnsafeUnpin for TryCapturesIter<'h, F>
impl<'h, F> UnwindSafe for TryCapturesIter<'h, F>
impl<I> IntoIterator for TryCapturesIter<'h, F>
fn into_iter(self: Self) -> I
impl<T> Any for TryCapturesIter<'h, F>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for TryCapturesIter<'h, F>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for TryCapturesIter<'h, F>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for TryCapturesIter<'h, F>
fn from(t: T) -> TReturns the argument unchanged.
impl<T, U> Into for TryCapturesIter<'h, F>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for TryCapturesIter<'h, F>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for TryCapturesIter<'h, F>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>