Struct TryHalfMatchesIter

struct TryHalfMatchesIter<'h, F> { ... }

An iterator over all non-overlapping half matches for a fallible search.

The iterator yields a Result<HalfMatch, MatchError> value until no more matches could be found.

The type parameters are as follows:

The lifetime parameters come from the Input type:

When possible, prefer the iterators defined on the regex engine you're using. This tries to abstract over the regex engine and is thus a bit more unwieldy to use.

This iterator is created by Searcher::into_half_matches_iter.

Implementations

impl<'h, F> TryHalfMatchesIter<'h, F>

fn infallible(self: Self) -> HalfMatchesIter<'h, F>

Return an infallible version of this iterator.

Any item yielded that corresponds to an error results in a panic. This is useful if your underlying regex engine is configured in a way that it is guaranteed to never return an error.

fn input<'i>(self: &'i Self) -> &'i Input<'h>

Returns the current Input used by this iterator.

The Input returned is generally equivalent to the one used to construct this iterator, but its start position may be different to reflect the start of the next search to be executed.

impl<'h, F> Debug for TryHalfMatchesIter<'h, F>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'h, F> Freeze for TryHalfMatchesIter<'h, F>

impl<'h, F> Iterator for TryHalfMatchesIter<'h, F>

fn next(self: &mut Self) -> Option<Result<HalfMatch, MatchError>>

impl<'h, F> RefUnwindSafe for TryHalfMatchesIter<'h, F>

impl<'h, F> Send for TryHalfMatchesIter<'h, F>

impl<'h, F> Sync for TryHalfMatchesIter<'h, F>

impl<'h, F> Unpin for TryHalfMatchesIter<'h, F>

impl<'h, F> UnsafeUnpin for TryHalfMatchesIter<'h, F>

impl<'h, F> UnwindSafe for TryHalfMatchesIter<'h, F>

impl<I> IntoIterator for TryHalfMatchesIter<'h, F>

fn into_iter(self: Self) -> I

impl<T> Any for TryHalfMatchesIter<'h, F>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for TryHalfMatchesIter<'h, F>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for TryHalfMatchesIter<'h, F>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for TryHalfMatchesIter<'h, F>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for TryHalfMatchesIter<'h, F>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for TryHalfMatchesIter<'h, F>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for TryHalfMatchesIter<'h, F>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>