Struct TryMatchesIter

struct TryMatchesIter<'h, F> { ... }

An iterator over all non-overlapping matches for a fallible search.

The iterator yields a Result<Match, MatchError> value until no more matches could be found.

The type parameters are as follows:

The lifetime parameters come from the Input type:

When possible, prefer the iterators defined on the regex engine you're using. This tries to abstract over the regex engine and is thus a bit more unwieldy to use.

This iterator is created by Searcher::into_matches_iter.

Implementations

impl<'h, F> TryMatchesIter<'h, F>

fn infallible(self: Self) -> MatchesIter<'h, F>

Return an infallible version of this iterator.

Any item yielded that corresponds to an error results in a panic. This is useful if your underlying regex engine is configured in a way that it is guaranteed to never return an error.

fn input<'i>(self: &'i Self) -> &'i Input<'h>

Returns the current Input used by this iterator.

The Input returned is generally equivalent to the one used to construct this iterator, but its start position may be different to reflect the start of the next search to be executed.

impl<'h, F> Debug for TryMatchesIter<'h, F>

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl<'h, F> Freeze for TryMatchesIter<'h, F>

impl<'h, F> Iterator for TryMatchesIter<'h, F>

fn next(self: &mut Self) -> Option<Result<Match, MatchError>>

impl<'h, F> RefUnwindSafe for TryMatchesIter<'h, F>

impl<'h, F> Send for TryMatchesIter<'h, F>

impl<'h, F> Sync for TryMatchesIter<'h, F>

impl<'h, F> Unpin for TryMatchesIter<'h, F>

impl<'h, F> UnsafeUnpin for TryMatchesIter<'h, F>

impl<'h, F> UnwindSafe for TryMatchesIter<'h, F>

impl<I> IntoIterator for TryMatchesIter<'h, F>

fn into_iter(self: Self) -> I

impl<T> Any for TryMatchesIter<'h, F>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for TryMatchesIter<'h, F>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for TryMatchesIter<'h, F>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for TryMatchesIter<'h, F>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for TryMatchesIter<'h, F>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for TryMatchesIter<'h, F>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for TryMatchesIter<'h, F>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>