pub struct PoolGuard<'a, T: Send, F: Fn() -> T>(/* private fields */);Expand description
A guard that is returned when a caller requests a value from the pool.
The purpose of the guard is to use RAII to automatically put the value back in the pool once it’s dropped.
Implementations