Struct StateIDError
struct StateIDError(_)
This error occurs when a value could not be constructed.
This occurs when given an integer exceeding the maximum allowed value.
When the std feature is enabled, this implements the Error
trait.
Implementations
impl StateIDError
fn attempted(self: &Self) -> u64Returns the value that could not be converted to an ID.
impl Clone for StateIDError
fn clone(self: &Self) -> StateIDError
impl Debug for StateIDError
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Display for StateIDError
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Eq for StateIDError
impl Error for StateIDError
impl Freeze for StateIDError
impl PartialEq for StateIDError
fn eq(self: &Self, other: &StateIDError) -> bool
impl RefUnwindSafe for StateIDError
impl Send for StateIDError
impl StructuralPartialEq for StateIDError
impl Sync for StateIDError
impl Unpin for StateIDError
impl UnsafeUnpin for StateIDError
impl UnwindSafe for StateIDError
impl<T> Any for StateIDError
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for StateIDError
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for StateIDError
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for StateIDError
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for StateIDError
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for StateIDError
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T> ToString for StateIDError
fn to_string(self: &Self) -> String
impl<T, U> Into for StateIDError
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for StateIDError
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for StateIDError
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>