Function parse_with
fn parse_with(pattern: &str, config: &Config) -> Result<Hir, Error>
A convenience routine for parsing a pattern into an HIR value using a
Config.
Example
This shows how to parse a pattern into an HIR value with a non-default configuration:
use syntax;
let hir = parse_with?;
assert!;
# Ok::