Struct BuildError
struct BuildError { ... }
An error that occurs when initial construction of a lazy DFA fails.
A build error can occur when insufficient cache capacity is configured or if something about the NFA is unsupported. (For example, if one attempts to build a lazy DFA without heuristic Unicode support but with an NFA that contains a Unicode word boundary.)
This error does not provide many introspection capabilities. There are generally only two things you can do with it:
- Obtain a human readable message via its
std::fmt::Displayimpl. - Access an underlying
nfa::thompson::BuildErrortype from itssourcemethod via thestd::error::Errortrait. This error only occurs when using convenience routines for building a lazy DFA directly from a pattern string.
When the std feature is enabled, this implements the std::error::Error
trait.
Implementations
impl Clone for BuildError
fn clone(self: &Self) -> BuildError
impl Debug for BuildError
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Display for BuildError
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Error for BuildError
fn source(self: &Self) -> Option<&dyn Error + 'static>
impl Freeze for BuildError
impl RefUnwindSafe for BuildError
impl Send for BuildError
impl Sync for BuildError
impl Unpin for BuildError
impl UnsafeUnpin for BuildError
impl UnwindSafe for BuildError
impl<T> Any for BuildError
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for BuildError
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for BuildError
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for BuildError
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for BuildError
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for BuildError
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T> ToString for BuildError
fn to_string(self: &Self) -> String
impl<T, U> Into for BuildError
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for BuildError
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for BuildError
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>