Struct Config
struct Config { ... }
A common set of configuration options that apply to the syntax of a regex.
This represents a group of configuration options that specifically apply
to how the concrete syntax of a regular expression is interpreted. In
particular, they are generally forwarded to the
ParserBuilder
in the
regex-syntax
crate when building a regex from its concrete syntax directly.
These options are defined as a group since they apply to every regex engine in this crate. Instead of re-defining them on every engine's builder, they are instead provided here as one cohesive unit.
Implementations
impl Config
fn new() -> ConfigReturn a new default syntax configuration.
fn case_insensitive(self: Self, yes: bool) -> ConfigEnable or disable the case insensitive flag by default.
When Unicode mode is enabled, case insensitivity is Unicode-aware. Specifically, it will apply the "simple" case folding rules as specified by Unicode.
By default this is disabled. It may alternatively be selectively enabled in the regular expression itself via the
iflag.fn multi_line(self: Self, yes: bool) -> ConfigEnable or disable the multi-line matching flag by default.
When this is enabled, the
^and$look-around assertions will match immediately after and immediately before a new line character, respectively. Note that the\Aand\zlook-around assertions are unaffected by this setting and always correspond to matching at the beginning and end of the input.By default this is disabled. It may alternatively be selectively enabled in the regular expression itself via the
mflag.fn dot_matches_new_line(self: Self, yes: bool) -> ConfigEnable or disable the "dot matches any character" flag by default.
When this is enabled,
.will match any character. When it's disabled, then.will match any character except for a new line character.Note that
.is impacted by whether the "unicode" setting is enabled or not. When Unicode is enabled (the default),.will match any UTF-8 encoding of any Unicode scalar value (sans a new line, depending on whether this "dot matches new line" option is enabled). When Unicode mode is disabled,.will match any byte instead. Because of this, when Unicode mode is disabled,.can only be used when the "allow invalid UTF-8" option is enabled, since.could otherwise match invalid UTF-8.By default this is disabled. It may alternatively be selectively enabled in the regular expression itself via the
sflag.fn crlf(self: Self, yes: bool) -> ConfigEnable or disable the "CRLF mode" flag by default.
By default this is disabled. It may alternatively be selectively enabled in the regular expression itself via the
Rflag.When CRLF mode is enabled, the following happens:
- Unless
dot_matches_new_lineis enabled,.will match any character except for\rand\n. - When
multi_linemode is enabled,^and$will treat\r\n,\rand\nas line terminators. And in particular, neither will match between a\rand a\n.
- Unless
fn line_terminator(self: Self, byte: u8) -> ConfigSets the line terminator for use with
(?u-s:.)and(?-us:.).Namely, instead of
.(by default) matching everything except for\n, this will cause.to match everything except for the byte given.If
.is used in a context where Unicode mode is enabled and this byte isn't ASCII, then an error will be returned. When Unicode mode is disabled, then any byte is permitted, but will return an error if UTF-8 mode is enabled and it is a non-ASCII byte.In short, any ASCII value for a line terminator is always okay. But a non-ASCII byte might result in an error depending on whether Unicode mode or UTF-8 mode are enabled.
Note that if
Rmode is enabled then it always takes precedence and the line terminator will be treated as\rand\nsimultaneously.Note also that this doesn't impact the look-around assertions
(?m:^)and(?m:$). That's usually controlled by additional configuration in the regex engine itself.fn swap_greed(self: Self, yes: bool) -> ConfigEnable or disable the "swap greed" flag by default.
When this is enabled,
.*(for example) will become ungreedy and.*?will become greedy.By default this is disabled. It may alternatively be selectively enabled in the regular expression itself via the
Uflag.fn ignore_whitespace(self: Self, yes: bool) -> ConfigEnable verbose mode in the regular expression.
When enabled, verbose mode permits insigificant whitespace in many places in the regular expression, as well as comments. Comments are started using
#and continue until the end of the line.By default, this is disabled. It may be selectively enabled in the regular expression by using the
xflag regardless of this setting.fn unicode(self: Self, yes: bool) -> ConfigEnable or disable the Unicode flag (
u) by default.By default this is enabled. It may alternatively be selectively disabled in the regular expression itself via the
uflag.Note that unless "allow invalid UTF-8" is enabled (it's disabled by default), a regular expression will fail to parse if Unicode mode is disabled and a sub-expression could possibly match invalid UTF-8.
WARNING: Unicode mode can greatly increase the size of the compiled DFA, which can noticeably impact both memory usage and compilation time. This is especially noticeable if your regex contains character classes like
\wthat are impacted by whether Unicode is enabled or not. If Unicode is not necessary, you are encouraged to disable it.fn utf8(self: Self, yes: bool) -> ConfigWhen disabled, the builder will permit the construction of a regular expression that may match invalid UTF-8.
For example, when
Config::unicodeis disabled, then expressions like[^a]may match invalid UTF-8 since they can match any single byte that is nota. By default, these sub-expressions are disallowed to avoid returning offsets that split a UTF-8 encoded codepoint. However, in cases where matching at arbitrary locations is desired, this option can be disabled to permit all such sub-expressions.When enabled (the default), the builder is guaranteed to produce a regex that will only ever match valid UTF-8 (otherwise, the builder will return an error).
fn nest_limit(self: Self, limit: u32) -> ConfigSet the nesting limit used for the regular expression parser.
The nesting limit controls how deep the abstract syntax tree is allowed to be. If the AST exceeds the given limit (e.g., with too many nested groups), then an error is returned by the parser.
The purpose of this limit is to act as a heuristic to prevent stack overflow when building a finite automaton from a regular expression's abstract syntax tree. In particular, construction currently uses recursion. In the future, the implementation may stop using recursion and this option will no longer be necessary.
This limit is not checked until the entire AST is parsed. Therefore, if callers want to put a limit on the amount of heap space used, then they should impose a limit on the length, in bytes, of the concrete pattern string. In particular, this is viable since the parser will limit itself to heap space proportional to the length of the pattern string.
Note that a nest limit of
0will return a nest limit error for most patterns but not all. For example, a nest limit of0permitsabut notab, sinceabrequires a concatenation AST item, which results in a nest depth of1. In general, a nest limit is not something that manifests in an obvious way in the concrete syntax, therefore, it should not be used in a granular way.fn octal(self: Self, yes: bool) -> ConfigWhether to support octal syntax or not.
Octal syntax is a little-known way of uttering Unicode codepoints in a regular expression. For example,
a,\x61,\u0061and\141are all equivalent regular expressions, where the last example shows octal syntax.While supporting octal syntax isn't in and of itself a problem, it does make good error messages harder. That is, in PCRE based regex engines, syntax like
\1invokes a backreference, which is explicitly unsupported in Rust's regex engine. However, many users expect it to be supported. Therefore, when octal support is disabled, the error message will explicitly mention that backreferences aren't supported.Octal syntax is disabled by default.
fn get_unicode(self: &Self) -> boolReturns whether "unicode" mode is enabled.
fn get_case_insensitive(self: &Self) -> boolReturns whether "case insensitive" mode is enabled.
fn get_multi_line(self: &Self) -> boolReturns whether "multi line" mode is enabled.
fn get_dot_matches_new_line(self: &Self) -> boolReturns whether "dot matches new line" mode is enabled.
fn get_crlf(self: &Self) -> boolReturns whether "CRLF" mode is enabled.
fn get_line_terminator(self: &Self) -> u8Returns the line terminator in this syntax configuration.
fn get_swap_greed(self: &Self) -> boolReturns whether "swap greed" mode is enabled.
fn get_ignore_whitespace(self: &Self) -> boolReturns whether "ignore whitespace" mode is enabled.
fn get_utf8(self: &Self) -> boolReturns whether UTF-8 mode is enabled.
fn get_nest_limit(self: &Self) -> u32Returns the "nest limit" setting.
fn get_octal(self: &Self) -> boolReturns whether "octal" mode is enabled.
impl Clone for Config
fn clone(self: &Self) -> Config
impl Copy for Config
impl Debug for Config
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Default for Config
fn default() -> Config
impl Freeze for Config
impl RefUnwindSafe for Config
impl Send for Config
impl Sync for Config
impl Unpin for Config
impl UnsafeUnpin for Config
impl UnwindSafe for Config
impl<T> Any for Config
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for Config
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for Config
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> CloneToUninit for Config
unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)
impl<T> From for Config
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToOwned for Config
fn to_owned(self: &Self) -> Tfn clone_into(self: &Self, target: &mut T)
impl<T, U> Into for Config
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for Config
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for Config
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>