Struct DeserializeError
struct DeserializeError(_)
An error that occurs when deserializing an object defined in this crate.
Serialization, as used in this crate, universally refers to the process
of transforming a structure (like a DFA) into a custom binary format
represented by &[u8]. Deserialization, then, refers to the process of
cheaply converting this binary format back to the object's in-memory
representation as defined in this crate. To the extent possible,
deserialization will report this error whenever this process fails.
A DeserializeError provides no introspection capabilities. Its only
supported operation is conversion to a human readable error message.
This error type implements the std::error::Error trait only when the
std feature is enabled. Otherwise, this type is defined in all
configurations.
Implementations
impl Debug for DeserializeError
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Display for DeserializeError
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl Error for DeserializeError
impl Freeze for DeserializeError
impl RefUnwindSafe for DeserializeError
impl Send for DeserializeError
impl Sync for DeserializeError
impl Unpin for DeserializeError
impl UnsafeUnpin for DeserializeError
impl UnwindSafe for DeserializeError
impl<T> Any for DeserializeError
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for DeserializeError
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for DeserializeError
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for DeserializeError
fn from(t: T) -> TReturns the argument unchanged.
impl<T> ToString for DeserializeError
fn to_string(self: &Self) -> String
impl<T, U> Into for DeserializeError
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for DeserializeError
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for DeserializeError
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>